/*
 * Based on Obdev's AVRUSB code and under the same license.
 *
 * SpaceOrb.h is a library for Arduino that wraps around the AVRUSB driver from ObDev
 *
 * Adapted from the UsbKeyboard.h library from Phil Lindsay
 *
 * by Michel Gutlich
 *
 */
#ifndef __SpaceOrb_h__
#define __SpaceOrb_h__

#include <avr/pgmspace.h>

#include "usbdrv.h"
/* these added for compatibility with 0012 Arduino */
#include <avr/interrupt.h>
#include <string.h>
typedef uint8_t byte;
#include <util/delay.h>


static uchar reportBuffer[10];				/* buffer for HID reports */

static uchar    idleRate;           // in 4 ms units 


PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {  // USB report descriptor for a 6-axis, 4 button joystick
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x04,                    // USAGE (Joystick)
    0xa1, 0x01,                    // COLLECTION (Application)
    0xa1, 0x00,                    //   COLLECTION (Physical)
    0x09, 0x30,                    //     USAGE (X)
    0x09, 0x31,                    //     USAGE (Y)
    0x09, 0x32,                    //     USAGE (Z)
    0x09, 0x33,                    //     USAGE (Rx)
    0x09, 0x34,                    //     USAGE (Ry)
    0x09, 0x35,                    //     USAGE (Rz)
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x03,              //     LOGICAL_MAXIMUM (1023)
    0x75, 0x0a,                    //     REPORT_SIZE (10)
    0x95, 0x06,                    //     REPORT_COUNT (6)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0xc0,                          //     END_COLLECTION
    0x05, 0x09,                    //   USAGE_PAGE (Button)
    0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
    0x29, 0x10,                    //   USAGE_MAXIMUM (Button 16)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x10,                    //   REPORT_COUNT (16)
    0x55, 0x00,                    //   UNIT_EXPONENT (0)
    0x65, 0x00,                    //   UNIT (None)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x75, 0x04,                    //   REPORT_SIZE (1)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x81, 0x03,                    //   INPUT (Cnst,Var,Abs)
    0xc0                           // END_COLLECTION
};

unsigned short reverse_word( unsigned short b );

class SpaceOrbDevice {
 public:
  SpaceOrbDevice () {
    
    usbInit();
      
    sei();

  }
    
  void refresh() {
    usbPoll();
  }
    
  void sendJoystick( unsigned short x,
		     unsigned short y,
		     unsigned short z,
		     unsigned short rx,
		     unsigned short ry,
		     unsigned short rz,
		     unsigned short btn )
    {
      unsigned char high;
      unsigned char low;
      unsigned char temp;
      //      unsigned short rbtn;
      high = x >> 8;
      low = x & 255;
      reportBuffer[0] = low;
      temp = high;

      high = y >> 6;
      low = y & 63;
      reportBuffer[1] = (low << 2) + temp;
      temp = high;

      high = z >> 4;
      low = y & 15;
      reportBuffer[2] = (low << 4) + temp;
      temp = high;
      
      high = rx >> 2;
      low = rx & 3;
      reportBuffer[3] = (low << 6) + temp;
      temp = high;

      high = 0;
      low = 0;
      reportBuffer[4] = temp;
      temp = high;

      high = ry >> 8;
      low = ry & 255;
      reportBuffer[5] = low+temp;
      temp = high;

      high = rz >> 6;
      low = rz & 63;
      reportBuffer[6] = (low << 2) + temp;
      temp = high;

      // now buttons.  This is tricky because logical_buttons
      // is an unsigned short from orb_buffer where button 0 is bit 0,
      // but HID wants button 0 to be the first bit sent
      //      rbtn = reverse_word(btn);
      reportBuffer[7] = (temp & 15 ) + ((btn & 15) << 4);
      reportBuffer[8] = (btn & 4080) >> 4; //(buttons1_8 & 15) + (buttons9_16 << 4);
      reportBuffer[9] = (btn >> 12);//0;//(rbtn & 15) << 4;//btn; //(buttons9_16&15);
      while (!usbInterruptIsReady())
	{
	  usbPoll();
	}
      if(usbInterruptIsReady()) {
	usbSetInterrupt(reportBuffer, 8 );
      }
      while (!usbInterruptIsReady())
	{
	  usbPoll();      
	}
      if (usbInterruptIsReady())
	{
	  usbSetInterrupt( reportBuffer+8, 2 );
	}
    }

  void sendRawPacket(byte val0, byte val1, byte val2, byte val3, byte val4 , byte val5 , byte val6, byte val7, byte val8, byte val9) {
      
    reportBuffer[0] = val0;
    reportBuffer[1] = val1;
    reportBuffer[2] = val2;
    reportBuffer[3] = val3;
    reportBuffer[4] = val4;
    reportBuffer[5] = val5;
    reportBuffer[6] = val6;
    reportBuffer[7] = val7;
    while (!usbInterruptIsReady())
      {
	usbPoll();
      }
    if(usbInterruptIsReady()) {
      usbSetInterrupt(reportBuffer, 8 );
    }
    while (!usbInterruptIsReady())
      {
	usbPoll();      
      }
    if (usbInterruptIsReady())
      {
	reportBuffer[0] = val8;
	reportBuffer[1] = val9;
	usbSetInterrupt( reportBuffer, 2 );
      }
  }
    
  //private: TODO: Make friend?
  uchar    reportBuffer[10];    // buffer for HID reports

};

SpaceOrbDevice SpaceOrb = SpaceOrbDevice();

#ifdef __cplusplus
extern "C"{
#endif 
	
  // USB_PUBLIC uchar usbFunctionSetup
uchar usbFunctionSetup(uchar data[8]) 
  {
	usbRequest_t    *rq = (usbRequest_t *)((void *)data);
	  
    usbMsgPtr = SpaceOrb.reportBuffer; //
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){ /* class request type */
      if(rq->bRequest == USBRQ_HID_GET_REPORT){ /* wValue: ReportType (highbyte), ReportID (lowbyte) */
		  reportBuffer[0]= 0;
		  reportBuffer[1]= 0;
		  reportBuffer[2]= 0;
		  reportBuffer[3]= 0;
		  reportBuffer[4]= 0;
		  reportBuffer[5]= 0;
		  reportBuffer[6]= 0;
		  reportBuffer[7]= 0;
	   
	return sizeof(reportBuffer);

      }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
		usbMsgPtr = &idleRate;
		return 1;
      }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
		idleRate = rq->wValue.bytes[1];
      }else{
		  /* no vendor specific requests implemented */
	}
    return 0;
  }
}
#ifdef __cplusplus
} // extern "C"
#endif


#endif // __SpaceOrb_h__
